%undefine _include_minidebuginfo
%undefine _strict_symbol_defs_build
%define _lto_cflags %{nil}
%bcond_with  debuginfod

Summary:         Dynamic analysis tools to detect memory or thread bugs and profile
Name:            valgrind
Version:         3.21.0
Release:         2%{?dist}
License:         GPLv2+
URL:             https://www.valgrind.org/
Source0:         https://sourceware.org/pub/valgrind/valgrind-%{version}.tar.bz2
Patch3001:       valgrind-3.9.0-cachegrind-improvements.patch
Patch3002:       valgrind-3.9.0-ldso-supp.patch
Patch3003:       valgrind-3.16.0-some-stack-protector.patch
Patch3004:       valgrind-3.16.0-some-Wl-z-now.patch
# From https://bugs.kde.org/show_bug.cgi?id=402833
Patch3005:       valgrind-3.21.0-no-memcpy-replace-check.patch
# From https://bugs.kde.org/show_bug.cgi?id=469768
Patch3006:       valgrind-3.21.0-Add-with-gdbscripts-dir.patch
# From https://bugs.kde.org/show_bug.cgi?id=460192
Patch3007:       valgrind-3.21.0-epoll_pwait2.patch
# From https://bugs.kde.org/show_bug.cgi?id=470121
Patch3008:       valgrind-3.21.0-callgrind_control-no-strict.patch
# From https://bugs.kde.org/show_bug.cgi?id=470520
Patch3009:       valgrind-3.21.0-realloc-again.patch
 

BuildRequires:   make automake autoconf glibc-devel gcc-c++
BuildRequires:   gdb procps docbook-dtds
%if %{with debuginfod}
BuildRequires:   elfutils-debuginfod-client
Recommends:      elfutils-debuginfod-client
%endif

ExclusiveArch:   x86_64 aarch64 ppc64le
%ifarch x86_64
%define valarch amd64
%define only_arch --enable-only64bit
%endif
%ifarch aarch64
%define valarch arm64
%define only_arch --enable-only64bit
%endif
%ifarch ppc64le
%define valarch ppc64le
%define only_arch --enable-only64bit
%endif

%description
Valgrind is an instrumentation framework for building dynamic analysis
tools. There are Valgrind tools that can automatically detect many
memory management and threading bugs, and profile your programs in
detail. You can also use Valgrind to build new tools. The Valgrind
distribution currently includes six production-quality tools: a memory
error detector (memcheck, the default tool), two thread error
detectors (helgrind and drd), a cache and branch-prediction profiler
(cachegrind), a call-graph generating cache and branch-prediction
profiler (callgrind), and a heap profiler (massif).


%package devel
Summary:         Development files for valgrind aware programs
Requires:        valgrind = %{version}-%{release}

%description devel
Header files and libraries for development of valgrind aware programs.


%prep
%autosetup -n %{name}-%{version} -p1


%build
./autogen.sh
# https://bugs.busybox.net/show_bug.cgi?id=8946
export CFLAGS="`echo " %{optflags} " | sed 's/ -fstack-protector\([-a-z]*\) / / g;s/ -O2 / /g;'`"
export LDFLAGS="`echo " %{build_ldflags} "    | sed 's/ -Wl,-z,now / / g;'`"
%configure --with-mpicc=/bin/false %{only_arch} GDB=%{_bindir}/gdb
%make_build


%install
%make_install
mkdir docs/installed
mv %{buildroot}%{_datadir}/doc/valgrind/* docs/installed/
rm -f docs/installed/*.ps

rm %{buildroot}%{_includedir}/valgrind/config.h
rm %{buildroot}%{_includedir}/valgrind/libvex*h
rm %{buildroot}%{_includedir}/valgrind/pub_tool_*h
rm -rf %{buildroot}%{_includedir}/valgrind/vki
rm %{buildroot}%{_libdir}/valgrind/*.a

chmod 644 $RPM_BUILD_ROOT%{_libexecdir}/valgrind/vgpreload*-%{valarch}-*so


%check
uname -a
rpm -q glibc gcc binutils || true
rpm -q gdb || true

LD_SHOW_AUXV=1 /bin/true
cat /proc/cpuinfo

./vg-in-place --error-exitcode=1 /bin/true --help

%make_build CFLAGS="" CXXFLAGS="" LDFLAGS="" check

export PYTHONCOERCECLOCALE=0
make regtest || :

MAX_LINES=20
diff_files=`find gdbserver_tests */tests -name '*.diff*' | sort`
if [ z"$diff_files" = z ] ; then
   echo "Congratulations, all tests passed!" >> diffs
else
   for i in $diff_files ; do
      echo "=================================================" >> diffs
      echo $i                                                  >> diffs
      echo "=================================================" >> diffs
      if [ `wc -l < $i` -le $MAX_LINES ] ; then
         cat $i                                                >> diffs
      else
         head -n $MAX_LINES $i                                 >> diffs
         echo "<truncated beyond $MAX_LINES lines>"            >> diffs
      fi
   done
fi
cat diffs


%files
%license COPYING COPYING.DOCS
%doc NEWS README_*
%doc docs/installed/html docs/installed/*.pdf
%{_bindir}/*
%dir %{_libexecdir}/valgrind
%{_libexecdir}/valgrind/*[^o]
%attr(0755,root,root) %{_libexecdir}/valgrind/vgpreload*-%{valarch}-*so
%{_mandir}/man1/*


%files devel
%dir %{_includedir}/valgrind
%{_includedir}/valgrind/valgrind.h
%{_includedir}/valgrind/callgrind.h
%{_includedir}/valgrind/drd.h
%{_includedir}/valgrind/helgrind.h
%{_includedir}/valgrind/memcheck.h
%{_includedir}/valgrind/dhat.h
%{_libdir}/pkgconfig/valgrind.pc


%changelog
* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.21.0-2
- Rebuilt for OpenCloudOS Stream 23.09

* Fri Jul 07 2023 rockerzhu <rockerzhu@tencent.com> - 3.21.0-1
- Upgrade to 3.21.0

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.20.0-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.20.0-2
- Rebuilt for OpenCloudOS Stream 23

* Thu Dec 15 2022 rockerzhu <rockerzhu@tencent.com> - 3.20.0-1
- Initial build
